"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestRunner = void 0;
const utils_1 = require("./utils");
const getTestCoverage_1 = require("./getTestCoverage");
const TestSessionScheduler_1 = require("./TestSessionScheduler");
const TestSessionManager_1 = require("./TestSessionManager");
const TestSessionStatus_1 = require("./TestSessionStatus");
class TestRunner extends utils_1.EventEmitter {
    constructor(config, testFiles) {
        super();
        this.sessions = new TestSessionManager_1.TestSessionManager();
        this.browserNames = [];
        this.favoriteBrowser = '';
        this.startTime = -1;
        this.testRun = -1;
        this.started = false;
        this.stopped = false;
        this.config = config;
        this.testFiles = testFiles;
        this.browsers = Array.isArray(config.browsers) ? config.browsers : [config.browsers];
        this.scheduler = new TestSessionScheduler_1.TestScheduler(config, this.browsers, this.sessions);
        this.sessions.on('session-status-updated', (session) => {
            if (session.status === TestSessionStatus_1.STATUS_FINISHED) {
                this.onSessionFinished(session);
            }
        });
    }
    async start() {
        var _a;
        try {
            if (this.started) {
                throw new Error('Cannot start twice.');
            }
            this.started = true;
            this.startTime = Date.now();
            this.browserNames = [];
            for (const browser of this.browsers) {
                const names = await browser.start(this.config);
                if (!Array.isArray(names) ||
                    names.length === 0 ||
                    names.some((n) => typeof n !== 'string')) {
                    throw new Error('Browser start must return an array of strings.');
                }
                this.browserNames.push(...names);
            }
            this.favoriteBrowser = (_a = this.browserNames.find((browserName) => {
                const n = browserName.toLowerCase();
                return n.includes('chrome') || n.includes('chromium') || n.includes('firefox');
            })) !== null && _a !== void 0 ? _a : this.browserNames[0];
            const createdSessions = utils_1.createTestSessions(this.browserNames, this.testFiles);
            for (const session of createdSessions) {
                this.sessions.add(session);
            }
            await this.config.server.start({
                config: this.config,
                sessions: this.sessions,
                runner: this,
                testFiles: this.testFiles,
            });
            this.runTests(this.sessions.all());
        }
        catch (error) {
            this.quit(error);
        }
    }
    async runTests(sessions) {
        if (this.stopped) {
            return;
        }
        const sessionsToRun = this.focusedTestFile
            ? Array.from(sessions).filter((f) => f.testFile === this.focusedTestFile)
            : Array.from(sessions);
        if (sessionsToRun.length === 0) {
            return;
        }
        if (this.browsers.length > 1) {
            // TODO: only pass sessions to browsers associated with it
            throw new Error('Multiple browsers are not yet supported');
        }
        try {
            this.testRun += 1;
            await this.scheduler.schedule(this.testRun, sessionsToRun);
            this.emit('test-run-started', { testRun: this.testRun, sessions: sessionsToRun });
        }
        catch (error) {
            this.quit(error);
        }
    }
    async stop() {
        if (this.stopped) {
            return;
        }
        this.emit('quit');
        this.stopped = true;
        const tasks = [];
        tasks.push(this.config.server.stop().catch((error) => {
            console.error(error);
        }));
        for (const browser of this.browsers) {
            tasks.push(browser.stop().catch((error) => {
                console.error(error);
            }));
        }
        await Promise.all(tasks);
    }
    startDebugFocusedTestFile() {
        if (!this.focusedTestFile) {
            throw new Error('Cannot debug without a focused test file.');
        }
        for (const session of this.sessions.forTestFile(this.focusedTestFile)) {
            // TODO: select browser for session
            this.browsers[0].startDebugSession(session);
        }
    }
    async quit(error) {
        if (error instanceof Error) {
            console.error('Error while running tests:');
            console.error(error);
            console.error('');
        }
        await this.stop();
        process.exit(1);
    }
    async onSessionFinished(session) {
        try {
            // TODO: find correct browser for session
            for (const browser of this.browsers) {
                browser.stopSession(session);
            }
            this.scheduler.runScheduled(this.testRun).catch((error) => {
                this.quit(error);
            });
            const finishedAll = Array.from(this.sessions.all()).every((s) => s.status === TestSessionStatus_1.STATUS_FINISHED);
            if (finishedAll) {
                let passedCoverage = true;
                let testCoverage = undefined;
                if (this.config.coverage) {
                    const coverageThreshold = typeof this.config.coverage === 'object' ? this.config.coverage.threshold : undefined;
                    testCoverage = getTestCoverage_1.getTestCoverage(this.sessions.all(), coverageThreshold);
                    passedCoverage = testCoverage.passed;
                }
                this.emit('test-run-finished', { testRun: this.testRun, testCoverage });
                if (!this.config.watch) {
                    setTimeout(async () => {
                        await this.stop();
                        const failed = !passedCoverage || Array.from(this.sessions.failed()).length > 0;
                        process.exit(failed ? 1 : 0);
                    });
                }
            }
        }
        catch (error) {
            this.quit(error);
        }
    }
}
exports.TestRunner = TestRunner;
//# sourceMappingURL=TestRunner.js.map