"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestScheduler = void 0;
const events_1 = require("events");
const TestSessionStatus_1 = require("./TestSessionStatus");
class TestScheduler extends events_1.EventEmitter {
    constructor(config, browsers, sessions) {
        super();
        this.config = config;
        this.browsers = browsers;
        this.sessions = sessions;
    }
    async schedule(testRun, sessionsToSchedule) {
        for (const session of sessionsToSchedule) {
            this.sessions.updateStatus(session, TestSessionStatus_1.STATUS_SCHEDULED);
        }
        return this.runScheduled(testRun);
    }
    runScheduled(testRun) {
        const scheduleTasks = [];
        const scheduledIt = this.sessions.forStatus(TestSessionStatus_1.STATUS_SCHEDULED);
        const runningCount = Array.from(this.sessions.forStatus(TestSessionStatus_1.STATUS_INITIALIZING, TestSessionStatus_1.STATUS_STARTED))
            .length;
        const count = this.config.concurrency - runningCount;
        for (let i = 0; i < count; i += 1) {
            const { done, value } = scheduledIt.next();
            if (done || !value) {
                break;
            }
            scheduleTasks.push(this.runSession(testRun, value));
        }
        return Promise.all(scheduleTasks);
    }
    async runSession(testRun, session) {
        this.sessions.update(Object.assign(Object.assign({}, session), { testRun, status: TestSessionStatus_1.STATUS_INITIALIZING }));
        let browserStartResponded = false;
        // browser should be started within the specified milliseconds
        setTimeout(() => {
            if (!browserStartResponded) {
                this.setSessionFailed(this.sessions.get(session.id), {
                    message: `Browser did not start after ${this.config.browserStartTimeout}ms.`,
                });
            }
        }, this.config.browserStartTimeout);
        try {
            // TODO: Select associated browser
            await this.browsers[0].startSession(session);
            // when the browser started, wait for session to ping back on time
            this.setSessionStartedTimeout(testRun, session.id);
        }
        catch (error) {
            this.setSessionFailed(session, error);
        }
        finally {
            browserStartResponded = true;
        }
    }
    setSessionFailed(session, error) {
        this.sessions.updateStatus(session, TestSessionStatus_1.STATUS_FINISHED, { passed: false, error });
    }
    setSessionStartedTimeout(testRun, sessionId) {
        setTimeout(() => {
            const session = this.sessions.get(sessionId);
            if (session.testRun !== testRun) {
                // session reloaded in the meantime
                return;
            }
            if (session.status === TestSessionStatus_1.STATUS_INITIALIZING) {
                this.setSessionFailed(session, {
                    message: `Did not receive a start signal from browser after ${this.config.sessionStartTimeout}ms.`,
                });
                return;
            }
            if (session.status === TestSessionStatus_1.STATUS_FINISHED) {
                // The session finished by now
                return;
            }
            this.setSessionFinishedTimeout(testRun, session.id);
        }, this.config.sessionStartTimeout);
    }
    setSessionFinishedTimeout(testRun, sessionId) {
        setTimeout(() => {
            const session = this.sessions.get(sessionId);
            if (session.testRun !== testRun) {
                // session reloaded in the meantime
                return;
            }
            if (session.status !== TestSessionStatus_1.STATUS_FINISHED) {
                this.setSessionFailed(session, {
                    message: `Browser did not finish within ${this.config.sessionStartTimeout}ms.`,
                });
            }
        }, this.config.sessionFinishTimeout);
    }
}
exports.TestScheduler = TestScheduler;
//# sourceMappingURL=TestSessionScheduler.js.map