"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Terminal = void 0;
const log_update_1 = __importDefault(require("log-update"));
const cli_cursor_1 = __importDefault(require("cli-cursor"));
const utils_1 = require("../utils");
const CLEAR_COMMAND = process.platform === 'win32' ? '\x1B[2J\x1B[0f' : '\x1B[2J\x1B[3J\x1B[H';
function buildLogString(entries, serverAddress) {
    const strings = [];
    for (const entry of entries) {
        let stringsToAdd;
        let indent;
        if (typeof entry === 'string') {
            stringsToAdd = entry.split('\n');
            indent = 0;
        }
        else {
            stringsToAdd = entry.text.split('\n');
            indent = entry.indent;
        }
        strings.push(...stringsToAdd.map((str) => `${' '.repeat(indent)}${str.replace(serverAddress, '')}`));
    }
    return strings.join('\n');
}
class Terminal extends utils_1.EventEmitter {
    constructor() {
        super(...arguments);
        this.originalFunctions = {};
        this.previousDynamic = [];
        this.started = false;
        this.isInteractive = process.stdout.isTTY;
    }
    start(serverAddress, watchMode) {
        this.serverAddress = new RegExp(serverAddress, 'g');
        // start off with an empty line
        console.log('');
        for (const key of Object.keys(console)) {
            if (typeof console[key] === 'function') {
                this.originalFunctions[key] = console[key];
                console[key] = new Proxy(console[key], {
                    apply: (target, thisArg, argArray) => {
                        // TODO: Remove this when fixed in EDS
                        if (argArray.some((arg) => typeof arg === 'string' && arg.startsWith('[BABEL]'))) {
                            return;
                        }
                        // when a console function is called, clear dynamic logs
                        log_update_1.default.clear();
                        // do regular console log
                        target.apply(thisArg, argArray);
                        // rerender dynamic logs
                        this.relogDynamic();
                    },
                });
            }
        }
        process.stdin.resume();
        process.stdin.setEncoding('utf8');
        process.stdin.on('data', (input) => {
            this.emit('input', input);
        });
        this.started = true;
    }
    observeDirectInput() {
        if (!this.isInteractive) {
            throw new Error('Cannot observe input in a non-interactive (TTY) terminal.');
        }
        if (typeof process.stdin.setRawMode === 'function') {
            process.stdin.setRawMode(true);
        }
        cli_cursor_1.default.hide();
    }
    observeConfirmedInput() {
        if (!this.isInteractive) {
            throw new Error('Cannot observe input in a non-interactive (TTY) terminal.');
        }
        if (typeof process.stdin.setRawMode === 'function') {
            process.stdin.setRawMode(false);
        }
        cli_cursor_1.default.show();
    }
    stop() {
        log_update_1.default.done();
        for (const [key, fn] of Object.entries(this.originalFunctions)) {
            console[key] = fn;
        }
        this.started = false;
    }
    clear() {
        process.stdout.write(CLEAR_COMMAND);
        this.relogDynamic();
    }
    logStatic(entriesOrEntry) {
        const entries = Array.isArray(entriesOrEntry) ? entriesOrEntry : [entriesOrEntry];
        if (entries.length === 0) {
            return;
        }
        console.log(buildLogString(entries, this.serverAddress));
    }
    logPendingUserInput(string) {
        process.stdout.write(string);
    }
    logDynamic(entriesOrEntry) {
        const entries = Array.isArray(entriesOrEntry) ? entriesOrEntry : [entriesOrEntry];
        if (!this.started) {
            return;
        }
        this.previousDynamic = entries;
        log_update_1.default(buildLogString(entries, this.serverAddress));
    }
    relogDynamic() {
        this.logDynamic(this.previousDynamic);
    }
}
exports.Terminal = Terminal;
//# sourceMappingURL=Terminal.js.map