"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestRunnerCli = exports.MENUS = void 0;
const chalk_1 = __importDefault(require("chalk"));
const getTestProgressReport_1 = require("./getTestProgressReport");
const Terminal_1 = require("./Terminal");
const getTestFileReport_1 = require("./getTestFileReport");
const getTestCoverageReport_1 = require("./getTestCoverageReport");
const getWatchCommands_1 = require("./getWatchCommands");
const getSelectFilesMenu_1 = require("./getSelectFilesMenu");
const TestSessionStatus_1 = require("../TestSessionStatus");
exports.MENUS = {
    OVERVIEW: 'overview',
    FOCUS_SELECT_FILE: 'focus',
    DEBUG_SELECT_FILE: 'debug',
};
const KEYCODES = {
    ENTER: '\r',
    ESCAPE: '\u001b',
    CTRL_C: '\u0003',
    CTRL_D: '\u0004',
};
class TestRunnerCli {
    constructor(config, runner) {
        this.config = config;
        this.runner = runner;
        this.terminal = new Terminal_1.Terminal();
        this.reportedFilesByTestRun = new Map();
        this.activeMenu = exports.MENUS.OVERVIEW;
        this.menuSucceededFiles = [];
        this.menuFailedFiles = [];
        this.sessions = runner.sessions;
        this.serverAddress = `${config.address}:${config.port}/`;
        if (config.watch && !this.terminal.isInteractive) {
            this.runner.quit(new Error('Cannot run watch mode in a non-interactive (TTY) terminal.'));
        }
    }
    start() {
        this.setupTerminalEvents();
        this.setupRunnerEvents();
        this.terminal.start(this.serverAddress, !!this.config.watch);
        if (this.config.watch) {
            this.terminal.clear();
        }
        this.logTestResults();
        this.logTestProgress();
        this.terminal.observeDirectInput();
    }
    setupTerminalEvents() {
        this.terminal.on('input', (key) => {
            if ([exports.MENUS.DEBUG_SELECT_FILE, exports.MENUS.FOCUS_SELECT_FILE].includes(this.activeMenu)) {
                const i = Number(key);
                if (!Number.isNaN(i)) {
                    this.focusTestFileNr(i);
                    return;
                }
            }
            switch (key.toUpperCase()) {
                case KEYCODES.CTRL_C:
                case KEYCODES.CTRL_D:
                case 'Q':
                    this.runner.quit();
                    return;
                case 'D':
                    if (this.activeMenu === exports.MENUS.OVERVIEW) {
                        if (this.runner.focusedTestFile) {
                            this.runner.startDebugFocusedTestFile();
                        }
                        else {
                            this.switchMenu(exports.MENUS.DEBUG_SELECT_FILE);
                        }
                    }
                    return;
                case 'F':
                    if (this.activeMenu === exports.MENUS.OVERVIEW) {
                        this.switchMenu(exports.MENUS.FOCUS_SELECT_FILE);
                    }
                    return;
                case KEYCODES.ESCAPE:
                    if (this.activeMenu === exports.MENUS.OVERVIEW && this.runner.focusedTestFile) {
                        this.runner.focusedTestFile = undefined;
                        this.logTestResults(true);
                        this.logTestProgress();
                    }
                    return;
                case KEYCODES.ENTER:
                    this.runner.runTests(this.sessions.all());
                    return;
                default:
                    return;
            }
        });
    }
    setupRunnerEvents() {
        this.sessions.on('session-status-updated', (session) => {
            if (this.activeMenu !== exports.MENUS.OVERVIEW) {
                return;
            }
            if (session.status === TestSessionStatus_1.STATUS_FINISHED) {
                this.logTestResult(session.testFile);
                this.logTestProgress();
            }
        });
        this.runner.on('test-run-started', ({ testRun }) => {
            if (this.activeMenu !== exports.MENUS.OVERVIEW) {
                return;
            }
            if (testRun !== 0 && this.config.watch) {
                this.terminal.clear();
            }
            this.logTestResults();
        });
        this.runner.on('test-run-finished', ({ testCoverage }) => {
            if (this.activeMenu !== exports.MENUS.OVERVIEW) {
                return;
            }
            if (testCoverage) {
                this.logTestCoverage(testCoverage);
            }
        });
        this.runner.on('quit', () => {
            this.reportEnd();
        });
    }
    focusTestFileNr(i) {
        var _a;
        const focusedTestFile = (_a = this.menuFailedFiles[i - 1]) !== null && _a !== void 0 ? _a : this.menuSucceededFiles[i - this.menuFailedFiles.length - 1];
        const debug = this.activeMenu === exports.MENUS.DEBUG_SELECT_FILE;
        if (focusedTestFile) {
            this.runner.focusedTestFile = focusedTestFile;
            this.switchMenu(exports.MENUS.OVERVIEW);
            if (debug) {
                this.runner.startDebugFocusedTestFile();
            }
        }
        else {
            this.terminal.clear();
            this.logSelectFilesMenu();
        }
    }
    logTestResults(force = false) {
        const { testFiles, focusedTestFile } = this.runner;
        if (focusedTestFile) {
            this.logTestResult(focusedTestFile, force);
        }
        else {
            for (const testFile of testFiles) {
                this.logTestResult(testFile, force);
            }
        }
    }
    logTestResult(testFile, force = false) {
        const { testRun, browserNames, favoriteBrowser } = this.runner;
        const sessionsForTestFile = Array.from(this.sessions.forTestFile(testFile));
        const allFinished = sessionsForTestFile.every((s) => s.status === TestSessionStatus_1.STATUS_FINISHED);
        if (!allFinished) {
            return;
        }
        let reportedFiles = this.reportedFilesByTestRun.get(testRun);
        if (!reportedFiles) {
            reportedFiles = new Set();
            this.reportedFilesByTestRun.set(testRun, reportedFiles);
        }
        if (force || !(reportedFiles === null || reportedFiles === void 0 ? void 0 : reportedFiles.has(testFile))) {
            reportedFiles.add(testFile);
            this.terminal.logStatic(getTestFileReport_1.getTestFileReport(testFile, browserNames, favoriteBrowser, this.serverAddress, sessionsForTestFile));
        }
    }
    logTestCoverage(testCoverage) {
        const threshold = typeof this.config.coverage === 'object' ? this.config.coverage.threshold : undefined;
        this.terminal.logStatic(getTestCoverageReport_1.getTestCoverageReport(testCoverage, threshold));
    }
    switchMenu(menu) {
        if (this.activeMenu === menu) {
            return;
        }
        this.activeMenu = menu;
        if (this.config.watch) {
            this.terminal.clear();
        }
        switch (menu) {
            case exports.MENUS.OVERVIEW:
                this.logTestResults(true);
                this.logTestProgress();
                this.terminal.observeDirectInput();
                break;
            case exports.MENUS.FOCUS_SELECT_FILE:
            case exports.MENUS.DEBUG_SELECT_FILE:
                this.logSelectFilesMenu();
                break;
            default:
                break;
        }
    }
    logSelectFilesMenu() {
        this.menuSucceededFiles = [];
        this.menuFailedFiles = [];
        for (const testFile of this.runner.testFiles) {
            if (Array.from(this.sessions.forTestFile(testFile)).every((t) => { var _a; return (_a = t.result) === null || _a === void 0 ? void 0 : _a.passed; })) {
                this.menuSucceededFiles.push(testFile);
            }
            else {
                this.menuFailedFiles.push(testFile);
            }
        }
        const selectFilesEntries = getSelectFilesMenu_1.getSelectFilesMenu(this.menuSucceededFiles, this.menuFailedFiles, this.activeMenu);
        this.terminal.logDynamic([]);
        this.terminal.logStatic(selectFilesEntries);
        this.terminal.logPendingUserInput(`Number of the file to ${this.activeMenu === exports.MENUS.FOCUS_SELECT_FILE ? 'focus' : 'debug'}: `);
        this.terminal.observeConfirmedInput();
    }
    logTestProgress(final = false) {
        const logStatic = this.config.staticLogging || !this.terminal.isInteractive;
        if (logStatic && !final) {
            return;
        }
        const entries = [];
        entries.push(...getTestProgressReport_1.getTestProgressReport(this.config, {
            browserNames: this.runner.browserNames,
            testRun: this.runner.testRun,
            testFiles: this.runner.testFiles,
            sessions: this.sessions,
            startTime: this.runner.startTime,
            focusedTestFile: this.runner.focusedTestFile,
        }));
        if (this.runner.focusedTestFile) {
            entries.push(`Focused on test file: ${chalk_1.default.cyanBright(this.runner.focusedTestFile)}\n`);
        }
        if (this.config.watch) {
            entries.push(...getWatchCommands_1.getWatchCommands(this.activeMenu, this.runner.focusedTestFile), '');
        }
        if (logStatic) {
            this.terminal.logStatic(entries);
        }
        else {
            this.terminal.logDynamic(entries);
        }
    }
    reportEnd() {
        this.logTestProgress(true);
        this.terminal.stop();
    }
}
exports.TestRunnerCli = TestRunnerCli;
//# sourceMappingURL=TestRunnerCli.js.map